from json.decoder import JSONDecodeError
import pandas as pd
from .utils import get_requests
from .utils import get_season_data_from_game_data
from .utils import get_range_seasons_data


def get_game_play_by_play_data(season: int, gamecode: int) -> pd.DataFrame:
    """
    A function that gets the play-by-play data of a particular game.

    Args:

        season (int): The start year of the season

        gamecode (int): The game-code of the game of interest.
            It can be found on Euroleague's website.

    Returns:

        pd.DataFrame: A dataframe with the play-by-play data of the game.
    """
    url = "https://live.euroleague.net/api/PlaybyPlay"
    params = {
        "gamecode": gamecode,
        "seasoncode": f"E{season}"
    }
    r = get_requests(url, params=params)

    try:
        data = r.json()
    except JSONDecodeError:
        raise ValueError(f"Game code, {gamecode}, did not return any data.")

    periods = [
        'FirstQuarter', 'SecondQuarter', 'ThirdQuarter', 'ForthQuarter',
        'ExtraTime'
    ]
    all_data = []
    for p, period in enumerate(periods):
        if data[period]:
            df = pd.json_normalize(data[period])
            df["PERIOD"] = p + 1
            all_data.append(df)

    play_by_play_df = pd.concat(all_data)
    play_by_play_df['CODETEAM'] = play_by_play_df['CODETEAM'].str.strip()
    play_by_play_df['PLAYER_ID'] = play_by_play_df['PLAYER_ID'].str.strip()
    play_by_play_df.insert(0, 'Season', season)
    play_by_play_df.insert(1, 'Gamecode', gamecode)
    return play_by_play_df


def get_hame_shot_data_single_season(season: int) -> pd.DataFrame:
    """
    A function that gets the play-by-play data of *all* games in a single
    season

    Args:

        season (int): The start year of the season

    Returns:

        pd.DataFrame: A dataframe with the play-by-play data of all games in a
            single season
    """
    data_df = get_season_data_from_game_data(
        season, get_game_play_by_play_data)
    return data_df


def get_game_shot_data_multiple_seasons(
    start_season: int, end_season: int
) -> pd.DataFrame:
    """
    A function that gets the play-by-play data of *all* games in a range of
    seasons

    Args:

        start_season (int): The start year of the start season

        end_season (int): The start year of the end season

    Returns:

        pd.DataFrame: A dataframe with the play-by-play data of all games
            in range of seasons
    """
    df = get_range_seasons_data(
        start_season, end_season, get_game_play_by_play_data)
    return df



from typing import Optional
import pandas as pd
from .utils import get_player_stats, get_player_stats_leaders


def get_player_stats_all_seasons(
    endpoint: str,
    phase_type_code: Optional[str] = None,
    statistic_mode: str = "PerGame"
) -> pd.DataFrame:
    """
    The players' stats for *all* seasons.

    Args:

        endpoint (str): The type of stats, available variables:
            - traditional
            - advanced
            - misc
            - scoring

        phase_type_code (Optional[str], optional): The phase of the season,
            available variables:
            - "RS" (regular season)
            - "PO" (play-off)
            - "FF" (final four)
            Defaults to None, which includes all phases.

        statistic_mode (str, optional): The aggregation of statistics,
            available variables:
             - PerGame
            - Accumulated
            - Per100Possesions
            Defaults to "PerGame".

    Returns:

        pd.DataFrame: A dataframe with the players' stats
    """
    params = {"SeasonMode": "All"}
    df = get_player_stats(endpoint, params, phase_type_code, statistic_mode)
    return df


def get_player_stats_single_season(
    endpoint: str,
    season: int,
    phase_type_code: str,
    statistic_mode: str
) -> pd.DataFrame:
    """
    The players' stats for a *single* season.

    Args:

        endpoint (str): The type of stats, available variables:
            - traditional
            - advanced
            - misc
            - scoring

        season (int): The start year of the season.

        phase_type_code (Optional[str], optional): The phase of the season,
            available variables:
            - "RS" (regular season)
            - "PO" (play-off)
            - "FF" (final four)
            Defaults to None, which includes all phases.

        statistic_mode (str, optional): The aggregation of statistics,
            available variables:
             - PerGame
            - Accumulated
            - Per100Possesions
            Defaults to "PerGame".

    Returns:

        pd.DataFrame: A dataframe with the players' stats
    """
    params = {
        "SeasonMode": "Single",
        "SeasonCode": f"E{season}",
    }
    df = get_player_stats(endpoint, params, phase_type_code, statistic_mode)
    return df


def get_player_stats_range_seasons(
    endpoint: str,
    start_season: int,
    end_season: int,
    phase_type_code: Optional[str] = None,
    statistic_mode: str = "PerGame"
) -> pd.DataFrame:
    """
    The players' stats for a range of seasons.

    Args:

        endpoint (str): The type of stats, available variables:
            - traditional
            - advanced
            - misc
            - scoring

        start_season (int): The start year of the first season in the range.

        end_season (int): The start year of teh last season in the range.

        phase_type_code (Optional[str], optional): The phase of the season,
            available variables:
            - "RS" (regular season)
            - "PO" (play-off)
            - "FF" (final four)
            Defaults to None, which includes all phases.

        statistic_mode (str, optional): The aggregation of statistics,
            available variables:
             - PerGame
            - Accumulated
            - Per100Possesions
            Defaults to "PerGame".

    Returns:

        pd.DataFrame: A dataframe with the players' stats
    """
    params = {
        "SeasonMode": "Range",
        "FromSeasonCode": f"E{start_season}",
        "ToSeasonCode": f"E{end_season}",
    }
    df = get_player_stats(endpoint, params, phase_type_code, statistic_mode)
    return df


def get_player_stats_leaders_all_seasons(
    stat_category: str,
    top_n: int = 200,
    phase_type_code: Optional[str] = None,
    statistic_mode: str = "PerGame",
    game_type: Optional[str] = None,
    position: Optional[str] = None
) -> pd.DataFrame:
    """
    Get the top leaders in a statistical category in all seasons

    Args:

        stat_category (str): The stat category. See function
            `utils.get_player_stats_leaders` for a list of available stats.

        top_n (int): The number of top N players to return.  Defaults to 200.

        phase_type_code (Optional[str], optional): The phase of the season,
            available variables:
            - "RS" (regular season)
            - "PO" (play-off)
            - "FF" (final four)
            Defaults to None, which includes all phases.

        statistic_mode (str, optional): The aggregation of statistics,
            available variables:
            - PerGame
            - Accumulated
            - Per100Possesions
            - PerGameReverse
            - AccumulatedReverse
            Defaults to "PerGame".

        game_type (Optional[str], optional): The type of games to draw the top
            stats from. Available values:
            - HomeGames
            - AwayGames
            - GamesWon
            - GamesLost
            Defaults to None, meaning all games

        position (Optional[str], optional): The position of the player to draw
            the top stats from. Available values:
            - Guards
            - Forwards
            - Centers
            - RisingStars
            Defaults to None, meaning all positions.

    Returns:

        pd.DataFrame: A dataframe with the top leading players and their stat
    """
    params = {"SeasonMode": "All"}
    df = get_player_stats_leaders(
        params,
        stat_category,
        top_n,
        phase_type_code,
        statistic_mode,
        game_type,
        position
    )
    return df


def get_player_stats_leaders_single_season(
    season: int,
    stat_category: str,
    top_n: int = 200,
    phase_type_code: Optional[str] = None,
    statistic_mode: str = "PerGame",
    game_type: Optional[str] = None,
    position: Optional[str] = None
) -> pd.DataFrame:
    """
    Get the top leaders in a statistical category in a single season

    Args:

        season (int): The start year of the season.

        stat_category (str): The stat category. See function
            `utils.get_player_stats_leaders` for a list of available stats.
        top_n (int): The number of top N players to return.  Defaults to 200.

        phase_type_code (Optional[str], optional): The phase of the season,
            available variables:
            - "RS" (regular season)
            - "PO" (play-off)
            - "FF" (final four)
            Defaults to None, which includes all phases.

        statistic_mode (str, optional): The aggregation of statistics,
            available variables:
            - PerGame
            - Accumulated
            - Per100Possesions
            - PerGameReverse
            - AccumulatedReverse
            Defaults to "PerGame".

        game_type (Optional[str], optional): The type of games to draw the top
            stats from. Available values:
            - HomeGames
            - AwayGames
            - GamesWon
            - GamesLost
            Defaults to None, meaning all games

        position (Optional[str], optional): The position of the player to draw
            the top stats from. Available values:
            - Guards
            - Forwards
            - Centers
            - RisingStars
            Defaults to None, meaning all positions.

    Returns:

        pd.DataFrame: A dataframe with the top leading players and their stat
    """
    params = {
        "SeasonMode": "Single",
        "SeasonCode": f"E{season}",
    }
    df = get_player_stats_leaders(
        params,
        stat_category,
        top_n,
        phase_type_code,
        statistic_mode,
        game_type,
        position
    )
    return df


def get_player_stats_leaders_range_seasons(
    start_season: int,
    end_season: int,
    stat_category: str,
    top_n: int = 200,
    phase_type_code: Optional[str] = None,
    statistic_mode: str = "PerGame",
    game_type: Optional[str] = None,
    position: Optional[str] = None
) -> pd.DataFrame:
    """
    Get the top leaders in a statistical category in a range of seasons

    Args:

        start_season (int): The start year of the first season in the range.

        stat_category (str): The stat category. See function
            `utils.get_player_stats_leaders` for a list of available stats.

        top_n (int): The number of top N players to return.  Defaults to 200.

        phase_type_code (Optional[str], optional): The phase of the season,
            available variables:
            - "RS" (regular season)
            - "PO" (play-off)
            - "FF" (final four)
            Defaults to None, which includes all phases.

        statistic_mode (str, optional): The aggregation of statistics,
            available variables:
            - PerGame
            - Accumulated
            - Per100Possesions
            - PerGameReverse
            - AccumulatedReverse
            Defaults to "PerGame".

        game_type (Optional[str], optional): The type of games to draw the top
            stats from. Available values:
            - HomeGames
            - AwayGames
            - GamesWon
            - GamesLost
            Defaults to None, meaning all games

        position (Optional[str], optional): The position of the player to draw
            the top stats from. Available values:
            - Guards
            - Forwards
            - Centers
            - RisingStars
            Defaults to None, meaning all positions.

    Returns:

        pd.DataFrame: A dataframe with the top leading players and their stat
    """
    params = {
        "SeasonMode": "Range",
        "FromSeasonCode": f"E{start_season}",
        "ToSeasonCode": f"E{end_season}",
    }
    df = get_player_stats_leaders(
        params,
        stat_category,
        top_n,
        phase_type_code,
        statistic_mode,
        game_type,
        position
    )
    return df



from typing import Callable, Optional, List
import logging
import requests
import pandas as pd

logging.basicConfig(encoding='utf-8', level=logging.INFO)
logger = logging.getLogger(__name__)


BASE_URL = "https://api-live.euroleague.net"
version = "v3"
competition = "E"
URL = f"{BASE_URL}/{version}/competitions/{competition}"


def make_season_game_url(
    season: int,
    game_code: int,
    endpoint: str
) -> str:
    """
    Concatenates the base URL and makes the game url.

    Args:

        season (int): The start year of the season

        game_code (int): The code of the game. Find the code from
            Euroleague's website

        endpoint (str): The endpoint of the API

    Returns:

        str: the full URL
    """
    FULL_URL = f"{URL}/seasons/E{season}/games/{game_code}/{endpoint}"
    return FULL_URL


def get_requests(
    url: str,
    params={},
    headers={"Accept": "application/json"}
) -> requests.models.Response:
    """
    A wrapper to `requests.get()` which handles unsuccesful requests too.

    Args:

        url (str): _description_

        params (dict, optional): The `params` variables in get requests.
            Defaults to {}.

        headers (dict, optional): the `header` variable in get requests.
            Defaults to {"Accept": "application/json"}.

    Raises:

        Requests Error: If get request was not succesful

    Returns:

        requests.models.Response: The response object.
    """
    r = requests.get(url, params=params, headers=headers)

    if r.status_code != 200:
        r.raise_for_status()

    return r


def get_game_data(season: int, game_code: int, endpoint: str) -> pd.DataFrame:
    """
    A wrapper function for getting game-level data.

    Args:

        season (int): The start year of the season

        game_code (int): The game code of the game of interest.
            Find the game code from Euroleague's website

        endpoint (str): The type of game data, available variables:
            - report
            - stats
            - teamsComparison

    Raises:

        ValueError: If input endpoint is not applicable.

    Returns:

        pd.DataFrame: A dataframe with the game data.
    """
    game_endpoints = ["report", "stats", "teamsComparison"]

    raise_error(endpoint, "Statistic type", game_endpoints, False)

    url_ = make_season_game_url(season, game_code, endpoint)
    r = get_requests(url_)

    data = r.json()
    df = pd.json_normalize(data)
    return df


def get_season_data_from_game_data(
    season: int,
    fun: Callable[[int, int], pd.DataFrame]
) -> pd.DataFrame:
    """
    A wrapper function for getting game data for all games in a single season.

    Args:

        season (int, optional): The start year of the season.

        fun (Callable[[int, int], pd.DataFrame]): A callable function that
            determines that type of data to be collected. Available values:
            - get_game_report
            - get_game_stats
            - get_game_teams_comparison

    Returns:

        pd.DataFrame: A dataframe with the game data.
    """
    data_list = []
    game_code = 0
    attempt = 0
    while True:
        game_code += 1
        try:
            shots_df = fun(season, game_code)
            attempt = 0
            data_list.append(shots_df)
        except ValueError:
            attempt += 1

        # Due to the ban of Russian teams from Euroleague in 2021
        # this is a hack for not breaking in the first
        # "empty" game, but only after 5 *concecutive*
        # "empty" games
        if attempt > 5:
            logger.debug(
                "No more available game data for this season, break and exit"
            )
            break

    if data_list:
        data_df = pd.concat(data_list, axis=0)
    else:
        data_df = pd.DataFrame([])
    return data_df


def get_range_seasons_data(
    start_season: int,
    end_season: int,
    fun: Callable[[int, int], pd.DataFrame]
) -> pd.DataFrame:
    """
    A wrapper function with the all game data in a range of seasons

    Args:

        season (int, optional): The start year of the season.

        end_season (int): The start year of teh end season

        fun (Callable[[int, int], pd.DataFrame]): A callable function that
            determines that type of data to be collected. Available values:
            - get_game_report
            - get_game_stats
            - get_game_teams_comparison

    Returns:

        pd.DataFrame: A dataframe with the game data
    """
    data = []
    for season in range(start_season, end_season + 1):
        data_df = get_season_data_from_game_data(season, fun)
        data.append(data_df)
    df = pd.concat(data)
    return df


def get_player_stats(
    endpoint: str,
    params={},
    phase_type_code: Optional[str] = None,
    statistic_mode: str = "PerGame"
) -> pd.DataFrame:
    """
    A wrapper function for getting the players' stats for
    - all seasons
    - a single season
    - a range of seasons

    Args:

        endpoint (str): The type of stats, available variables:
            - traditional
            - advanced
            - misc
            - scoring

        params (Dict[str, Union[str, int]]): A dictionary of parameters for the
            get request.

        phase_type_code (Optional[str], optional): The phase of the season,
            available variables:
            - "RS" (regular season)
            - "PO" (play-off)
            - "FF" (final four)
            Defaults to None, which includes all phases.

        statistic_mode (str, optional): The aggregation of statistics,
            available variables:
            - PerGame
            - Accumulated
            - Per100Possesions
            Defaults to "PerGame".

    Raises:

        ValueError: If the endpoint is not applicable

        ValueError: If the phase_type_code is not applicable

        ValueError: If the statistic_mode is not applicable

    Returns:

        pd.DataFrame: A dataframe with the players' stats.
    """

    available_endpoints = ["traditional", "advanced", "misc", "scoring"]
    available_phase_type_code = ["RS", "PO", "FF"]
    available_stat_mode = ["PerGame", "Accumulated", "Per100Possesions"]

    raise_error(
        endpoint, "Statistic type", available_endpoints, False)
    raise_error(
        statistic_mode, "Statistic Aggregation", available_stat_mode, False)
    raise_error(
        phase_type_code, "Phase type code", available_phase_type_code, True)

    params["statisticMode"] = statistic_mode
    params["phaseTypeCode"] = phase_type_code
    params["limit"] = 400

    url_ = f"{URL}/statistics/players/{endpoint}"

    r = get_requests(url_, params=params)
    data = r.json()
    if data["total"] < len(data["players"]):
        params["limit"] = len(data["players"]) + 1
        r = get_requests(url_, params=params)
        data = r.json()
    df = pd.json_normalize(data["players"])
    return df


def get_team_stats(
    endpoint: str,
    params={},
    phase_type_code: Optional[str] = None,
    statistic_mode: str = "PerGame"
) -> pd.DataFrame:
    """
    A wrapper functions for collecting teams' stats.
    Allows for three types of data:
    - all seasons
    - single season
    - range of seasons

    Args:

        endpoint (str): The type of stats to fetch. Available values:
            - traditional
            - advanced
            - opponentsTraditional
            - opponentsAdvanced

        params (Dict[str, Union[str, int]]): A dictionary of the parmaters for
            the get request.

        phase_type_code (Optional[str], optional): The phase of the season,
            available variables:
            - "RS" (regular season)
            - "PO" (play-off)
            - "FF" (final four)
            Defaults to None, which includes all phases.

        statistic_mode (str, optional): The aggregation of statistics,
            available variables:
            - PerGame
            - Accumulated
            Defaults to "PerGame".

    Raises:

        ValueError: If the endpoint is not applicable

        ValueError: If the phase_type_code is not applicable

        ValueError: If the statistic_mode is not applicable

    Returns:

        pd.DataFrame: A dataframe with the teams' stats.
    """

    available_endpoints = [
        "traditional", "advanced",
        "opponentsTraditional", "opponentsAdvanced"
    ]
    available_phase_type_code = ["RS", "PO", "FF"]
    available_stat_mode = ["PerGame", "Accumulated"]

    raise_error(
        endpoint, "Statistic type", available_endpoints, False)
    raise_error(
        statistic_mode, "Statistic Aggregation", available_stat_mode, False)
    raise_error(
        phase_type_code, "Phase type code", available_phase_type_code, True)

    params["statisticMode"] = statistic_mode
    params["phaseTypeCode"] = phase_type_code
    params["limit"] = 400

    url_ = f"{URL}/statistics/teams/{endpoint}"

    r = get_requests(url_, params=params)
    data = r.json()
    if data["total"] < len(data["teams"]):
        params["limit"] = len(data["teams"]) + 1
        r = get_requests(url_, params=params)
        data = r.json()
    df = pd.json_normalize(data["teams"])
    return df


def get_player_stats_leaders(
    params={},
    stat_category: str = "Score",
    top_n: int = 200,
    phase_type_code: Optional[str] = None,
    statistic_mode: str = "PerGame",
    game_type: Optional[str] = None,
    position: Optional[str] = None
) -> pd.DataFrame:
    """
    A wrapper function for collecting the leading players in a given
    stat category.

    Args:

        params (Dict[str, Union[str, int]]): A dictionary of parameters for the
            get request.

        top_n (int): The number of top N players to return.  Defaults to 200.

        stat_category (str): The stat category. Available values:
            - Valuation
            - Score
            - TotalRebounds
            - OffensiveRebounds
            - Assistances
            - Steals
            - BlocksFavour
            - BlocksAgainst
            - Turnovers
            - FoulsReceived
            - FoulsCommited
            - FreeThrowsMade
            - FreeThrowsAttempted
            - FreeThrowsPercent
            - FieldGoalsMade2
            - FieldGoalsAttempted2
            - FieldGoals2Percent
            - FieldGoalsMade3
            - FieldGoalsAttempted3
            - FieldGoals3Percent
            - FieldGoalsMadeTotal
            - FieldGoalsAttemptedTotal
            - FieldGoalsPercent
            - AccuracyMade
            - AccuracyAttempted
            - AccuracyPercent
            - AssitancesTurnoversRation
            - GamesPlayed
            - GamesStarted
            - TimePlayed
            - Contras
            - Dunks
            - OffensiveReboundPercentage
            - DefensiveReboundPercentage
            - ReboundPercentage
            - EffectiveFeildGoalPercentage
            - TrueShootingPercentage
            - AssistRatio
            - TurnoverRatio
            - FieldGoals2AttemptedRatio
            - FieldGoals3AttemptedRatio
            - FreeThrowRate
            - Possessions
            - GamesWon
            - GamesLost
            - DoubleDoubles
            - TripleDoubles
            - FieldGoalsAttempted2Share
            - FieldGoalsAttempted3Share
            - FreeThrowsAttemptedShare
            - FieldGoalsMade2Share
            - FieldGoalsMade3Share
            - FreeThrowsMadeShare
            - PointsMade2Rate
            - PointsMade3Rate
            - PointsMadeFreeThrowsRate
            - PointsAttempted2Rate
            - PointsAttempted3Rate
            - Age

        phase_type_code (Optional[str], optional): The phase of the season,
            available variables:
            - "RS" (regular season)
            - "PO" (play-off)
            - "FF" (final four)
            Defaults to None, which includes all phases.

        statistic_mode (str, optional): The aggregation of statistics,
            available variables:
            - PerGame
            - Accumulated
            - Per100Possesions
            - PerGameReverse
            - AccumulatedReverse
            Defaults to "PerGame".

        game_type (Optional[str], optional): The type of games to draw the top
            stats from. Available values:
            - HomeGames
            - AwayGames
            - GamesWon
            - GamesLost
            Defaults to None, meaning all games

        position (Optional[str], optional): The position of the player to draw
            the top stats from. Available values:
            - Guards
            - Forwards
            - Centers
            - RisingStars
            Defaults to None, meaning all positions.

    Raises:

        ValueError: If the stat_category is not applicable

        ValueError: If the phase_type_code is not applicable

        ValueError: If the statistic_mode is not applicable

        ValueError: If the game_type is not applicable

        ValueError: If the position is not applicable

    Returns:

        pd.DataFrame: A dataframe with the top players' stats
    """
    avaiable_stat_category = [
        "Valuation",
        "Score",
        "TotalRebounds",
        "OffensiveRebounds",
        "Assistances",
        "Steals",
        "BlocksFavour",
        "BlocksAgainst",
        "Turnovers",
        "FoulsReceived",
        "FoulsCommited",
        "FreeThrowsMade",
        "FreeThrowsAttempted",
        "FreeThrowsPercent",
        "FieldGoalsMade2",
        "FieldGoalsAttempted2",
        "FieldGoals2Percent",
        "FieldGoalsMade3",
        "FieldGoalsAttempted3",
        "FieldGoals3Percent",
        "FieldGoalsMadeTotal",
        "FieldGoalsAttemptedTotal",
        "FieldGoalsPercent",
        "AccuracyMade",
        "AccuracyAttempted",
        "AccuracyPercent",
        "AssitancesTurnoversRation",
        "GamesPlayed",
        "GamesStarted",
        "TimePlayed",
        "Contras",
        "Dunks",
        "OffensiveReboundPercentage",
        "DefensiveReboundPercentage",
        "ReboundPercentage",
        "EffectiveFeildGoalPercentage",
        "TrueShootingPercentage",
        "AssistRatio",
        "TurnoverRatio",
        "FieldGoals2AttemptedRatio",
        "FieldGoals3AttemptedRatio",
        "FreeThrowRate",
        "Possessions",
        "GamesWon",
        "GamesLost",
        "DoubleDoubles",
        "TripleDoubles",
        "FieldGoalsAttempted2Share",
        "FieldGoalsAttempted3Share",
        "FreeThrowsAttemptedShare",
        "FieldGoalsMade2Share",
        "FieldGoalsMade3Share",
        "FreeThrowsMadeShare",
        "PointsMade2Rate",
        "PointsMade3Rate",
        "PointsMadeFreeThrowsRate",
        "PointsAttempted2Rate",
        "PointsAttempted3Rate",
        "Age"
    ]
    available_phase_type_code = ["RS", "PO", "FF"]
    available_stat_mode = [
        "PerGame",
        "Accumulated",
        "Per100Possesions",
        "PerGameReverse",
        "AccumulatedReverse"
    ]
    available_game_types = [
        "HomeGames",
        "AwayGames",
        "GamesWon",
        "GamesLost",
    ]
    availabe_positions = [
        "Guards",
        "Forwards",
        "Centers",
        "RisingStars"
    ]

    raise_error(stat_category, "Stat category", avaiable_stat_category, False)
    raise_error(
        statistic_mode, "Statistic Aggregation", available_stat_mode, False)
    raise_error(
        phase_type_code, "Phase type code", available_phase_type_code, True)
    raise_error(game_type, "Game type", available_game_types, True)
    raise_error(position, "Position", availabe_positions, True)

    params["category"] = stat_category
    params["phaseTypeCode"] = phase_type_code
    params["statisticMode"] = statistic_mode
    params["limit"] = top_n
    if game_type is not None and position is not None:
        raise ValueError(
            "Cannot select a game_type and position at the same type. "
            "One of the variables must be None"
        )
    elif game_type is not None and position is None:
        params["misc"] = game_type
    elif game_type is None and position is not None:
        params["misc"] = position

    url_ = f"{URL}/stats/players/leaders"

    r = get_requests(url_, params=params)
    data = r.json()
    df = pd.json_normalize(data["data"])
    return df


def raise_error(
    var: Optional[str],
    descripitve_var: str,
    available_vals: List,
    allow_none: bool = False,
) -> None:
    """
    A function that raises a ValueError with specific message.

    Args:

        var (str): The input variable by the user

        descripitve_var (str): A description in plain English of this variable

        available_vals (List): The available variables

        allow_none (bool, optional): If `var` can take None value.
            Defaults to False.

    Raises:

        ValueError: if `var` not applicable
    """

    if allow_none:
        available_vals.append(None)

    if var not in available_vals:
        raise ValueError(
            f"{descripitve_var}, {var}, is not applicable. "
            f"Available values: {available_vals}"
        )
    return



import pandas as pd
from .utils import get_season_data_from_game_data
from .utils import get_range_seasons_data
from .utils import get_game_data


def get_game_report(season: int, game_code: int) -> pd.DataFrame:
    """
    Get game report data for a game

    Args:
        season (int): The start year of the season
        game_code (int): The game code of the game of interest. It can be found
            on Euroleague's website.

    Returns:
        pd.DataFrame: A dataframw with the game report data
    """
    df = get_game_data(season, game_code, "report")
    return df


def get_game_reports_single_season(season: int) -> pd.DataFrame:
    """
    Get game report data for *all* games in a single season

    Args:
        season (int): The start year of the season

    Returns:
        pd.DataFrame: A dataframe with game report data
    """
    data_df = get_season_data_from_game_data(season, get_game_report)
    return data_df


def get_game_reports_range_seasons(
    start_season: int,
    end_season: int
) -> pd.DataFrame:
    """
    Get game report data for *all* games in a range of seasons

    Args:

        start_season (int): The start year of the start season

        end_season (int): The start year of the end season

    Returns:

        pd.DataFrame: A dataframe with game report data
    """
    df = get_range_seasons_data(start_season, end_season, get_game_report)
    return df


def get_game_stats(season: int, game_code: int) -> pd.DataFrame:
    """
    Get game stats data for single game

    Args:

        season (int): The start year of the season

        game_code (int): The game code of the game of interest. It can be found
            on Euroleague's website.

    Returns:

        pd.DataFrame: A dataframe with the games' stats data
    """
    df = get_game_data(season, game_code, "stats")
    return df


def get_game_stats_single_season(season: int) -> pd.DataFrame:
    """
    Get game stats data for *all* games in a single season

    Args:

        season (int): The start year of the season

    Returns:

        pd.DataFrame: A dataframe with the games' stats data
    """
    data_df = get_season_data_from_game_data(season, get_game_stats)
    return data_df


def get_game_stats_range_seasons(
    start_season: int,
    end_season: int
) -> pd.DataFrame:
    """
    Get game stats data for *all* games in a range of seasons

    Args:

        start_season (int): The start year of the start season

        end_season (int): The start year of the end season

    Returns:

        pd.DataFrame: A dataframe with the games' stats data
    """
    df = get_range_seasons_data(start_season, end_season, get_game_stats)
    return df


def get_game_teams_comparison(
    season: int,
    game_code: int
) -> pd.DataFrame:
    """
    A function that gets the "teams comparison" game stats for a single game

    Args:

        season (int): The start year of the season

        game_code (int): The game code of the game of interest. It can be found
            on Euroleague's website.

    Returns:

        pd.DataFrame: A dataframe with games teams comparison stats
    """
    df = get_game_data(season, game_code, "teamsComparison")
    return df


def get_game_teams_comparison_single_season(season: int) -> pd.DataFrame:
    """
    A function that gets the "teams comparison" game stats for *all* games
    in a single season

    Args:

        season (int): The start year of the season

    Returns:

        pd.DataFrame: A dataframe with games teams comparison stats
    """
    data_df = get_season_data_from_game_data(
        season, get_game_teams_comparison)
    return data_df


def get_game_teams_comparison_range_seasons(
    start_season: int,
    end_season: int
) -> pd.DataFrame:
    """
    A function that gets the "teams comparison" game stats for *all* in a
    range seasons

    Args:

        start_season (int): The start year of the star season

        end_season (int): The start year of the end season

    Returns:

        pd.DataFrame: A dataframe with games teams comparison stats
    """
    df = get_range_seasons_data(
        start_season,
        end_season,
        get_game_teams_comparison
    )
    return df



from typing import Optional
import pandas as pd
from .utils import get_team_stats


def get_team_stats_all_seasons(
    endpoint: str,
    phase_type_code: Optional[str] = None,
    statistic_mode: str = "PerGame"
) -> pd.DataFrame:
    """
    A function that gets the team stats for all seasons

    Args:

        endpoint (str): The type of stats to fetch. Available values:
            - traditional
            - advanced
            - opponentsTraditional
            - opponentsAdvanced

        phase_type_code (Optional[str], optional): The phase of the season,
            available variables:
            - "RS" (regular season)
            - "PO" (play-off)
            - "FF" (final four)
            Defaults to None, which includes all phases.

        statistic_mode (str, optional): The aggregation of statistics,
            available variables:
            - PerGame
            - Accumulated
            Defaults to "PerGame".

    Returns:

        pd.DataFrame: A dataframe with the teams' stats
    """
    params = {"SeasonMode": "All"}
    df = get_team_stats(endpoint, params, phase_type_code, statistic_mode)
    return df


def get_team_stats_single_season(
    endpoint: str,
    season: int,
    phase_type_code: str,
    statistic_mode: str
) -> pd.DataFrame:
    """
    A function that returns the teams' stats in a single season

    Args:

        endpoint (str): The type of stats to fetch. Available values:
            - traditional
            - advanced
            - opponentsTraditional
            - opponentsAdvanced

        season (int): The start year of the season

        phase_type_code (Optional[str], optional): The phase of the season,
            available variables:
            - "RS" (regular season)
            - "PO" (play-off)
            - "FF" (final four)
            Defaults to None, which includes all phases.

        statistic_mode (str, optional): The aggregation of statistics,
            available variables:
            - PerGame
            - Accumulated
            Defaults to "PerGame".

    Returns:

        pd.DataFrame: A dataframe with the teams' stats
    """
    params = {
        "SeasonMode": "Single",
        "SeasonCode": f"E{season}",
    }
    df = get_team_stats(endpoint, params, phase_type_code, statistic_mode)
    return df


def get_team_stats_range_seasons(
    endpoint: str,
    from_season: int,
    to_season: int,
    phase_type_code: Optional[str] = None,
    statistic_mode: str = "PerGame"
) -> pd.DataFrame:
    """
    A function that returns the teams' stats in a range of seasons

    Args:

        endpoint (str): The type of stats to fetch. Available values:
            - traditional
            - advanced
            - opponentsTraditional
            - opponentsAdvanced

        from_season (int): The start year of the start season

        to_season (int): The end year of the end season

        phase_type_code (Optional[str], optional): The phase of the season,
            available variables:
            - "RS" (regular season)
            - "PO" (play-off)
            - "FF" (final four)
            Defaults to None, which includes all phases.

        statistic_mode (str, optional): The aggregation of statistics,
            available variables:
            - PerGame
            - Accumulated
            Defaults to "PerGame".

    Returns:

        pd.DataFrame: A dataframe with the teams' stats
    """
    params = {
        "SeasonMode": "Range",
        "FromSeasonCode": f"E{from_season}",
        "ToSeasonCode": f"E{to_season}",
    }
    df = get_team_stats(endpoint, params, phase_type_code, statistic_mode)
    return df



import pandas as pd
from json.decoder import JSONDecodeError
from .utils import get_requests
from .utils import get_season_data_from_game_data
from .utils import get_range_seasons_data

MADE_ACTIONS = ['2FGM', '3FGM', 'LAYUPMD', 'DUNK']
MISSES_ACTIONS = ['2FGA', '2FGAB', '3FGA', '3FGAB', 'LAYUPATT']


def get_game_shot_data(season: int, gamecode: int) -> pd.DataFrame:
    """
    A function that gets the shot data of a particular game.

    Args:

        season (int): The start year of the season

        gamecode (int): The game-code of the game of interest.
            It can be found on Euroleague's website.

    Returns:

        pd.DataFrame: A dataframe with the shot data of the game.
    """
    url = "https://live.euroleague.net/api/Points"
    params = {
        "gamecode": gamecode,
        "seasoncode": f"E{season}"
    }
    r = get_requests(url, params=params)

    try:
        data = r.json()
    except JSONDecodeError:
        raise ValueError(f"Game code, {gamecode}, did not return any data.")

    shots_df = pd.DataFrame(data['Rows'])
    # team id, player id and action id contain trailing white space
    shots_df['TEAM'] = shots_df['TEAM'].str.strip()
    shots_df['ID_PLAYER'] = shots_df['ID_PLAYER'].str.strip()
    shots_df['ID_ACTION'] = shots_df['ID_ACTION'].str.strip()
    shots_df.insert(0, 'Season', season)
    shots_df.insert(1, 'Gamecode', gamecode)
    return shots_df


def get_game_shot_data_single_season(season: int) -> pd.DataFrame:
    """
    A function that gets the shot data of *all* games in a single season

    Args:

        season (int): The start year of the season

    Returns:

        pd.DataFrame: A dataframe with the shot data of all games in a single
            season
    """
    data_df = get_season_data_from_game_data(season, get_game_shot_data)
    return data_df


def get_game_shot_data_multiple_seasons(
    start_season: int, end_season: int
) -> pd.DataFrame:
    """
    A function that gets the shot data of *all* games in a range of seasons

    Args:

        start_season (int): The start year of the start season

        end_season (int): The start year of the end season

    Returns:

        pd.DataFrame: A dataframe with the shot data of all games in range of
            seasons
    """
    df = get_range_seasons_data(
        start_season, end_season, get_game_shot_data)
    return df



import pandas as pd
from .utils import (
    get_requests,
    URL
)


def get_standings(
    season: int,
    round_number: int,
    endpoint: str = "basicstandings",
) -> pd.DataFrame:
    """
    Get the standings of round in given season

    Args:

        season (int): The start year of the season

        round_number (int): The round number

        endpoint (str, optional): The type of standing.
        One of the following options
        - calendarstandings
        - streaks
        - aheadbehind
        - margins
        - basicstandings
        Defaults to "basicstandings".

    Raises:

        ValueError: If endpoint is not applicable

    Returns:

        pd.DataFrame: A dataframe with the standings of the teams
    """
    available_endpoints = [
        "calendarstandings", "streaks",
        "aheadbehind", "margins",
        "basicstandings"
    ]

    if endpoint not in available_endpoints:
        raise ValueError(
            "Standings endpoint, {endpoint}, is not applicable. Choose one of "
            f"the following: {available_endpoints}"
        )

    url_ = f"{URL}/seasons/E{season}/rounds/{round_number}/{endpoint}"
    r = get_requests(url_)
    data = r.json()
    df = pd.json_normalize(data["teams"])
    return df
